/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "AgentsWindow.hh"
#include "GabberApp.hh"

#include <discoDB.hh>

using namespace Gabber;

AgentsWindow::AgentsWindow(GabberApp& app) : 
    BaseGabberWindow("Agents_win"), _app(app)
{
    _app.getSession().discoDB().cache("jabber.org", SigC::slot(*this, &AgentsWindow::on_disco_result));
}

void AgentsWindow::on_disco_result(const jabberoo::DiscoDB::Item* item)
{
    // XXX This shouldn't happen?
    if (!item)
    {
        std::cout << "No disco result..." << std::endl;
        return;
    }

    std::cout << "RECEIVED DISCO RESULT: " << item->getJID() << std::endl;
    std::cout << "CHILDREN: " << std::endl;
    for (jabberoo::DiscoDB::Item::const_iterator it = item->begin(); 
         it != item->end(); ++it)
    {
        std::cout << "\t" << (*it)->getJID() << " -- " << (*it)->getName() << std::endl;
        const jabberoo::DiscoDB::Item::FeatureList& fl = (*it)->getFeatureList();
        for (jabberoo::DiscoDB::Item::FeatureList::const_iterator i = fl.begin();
             i != fl.end(); ++i)
        {
            std::cout << "\t\t" << (*i) << std::endl;
        }
    }
    std::cout << "IDENTITIES: " << std::endl;
    const jabberoo::DiscoDB::Item::IdentityList& il = item->getIdentityList();
    for (jabberoo::DiscoDB::Item::IdentityList::const_iterator i = il.begin(); 
         i != il.end(); ++i)
    {
        std::cout << "\t" << (*i)->getCategory() << ", " << (*i)->getType() << std::endl;
    }
}

