/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2003 Dave Smith & Julian Missig
 *  Copyright (c) 2002-2003 Julian Missig
 */

#ifndef INCL_FILETRANSFERMANAGER_HH
#define INCL_FILETRANSFERMANAGER_HH

#include "Stream.hh"
#include "FTProfile.hh"

#include <sigc++/object.h>
#include <jabberoo/discoDB.hh>

#include <list>
#include <string>
#include <map>
#include <fstream>

namespace Gabber {

class FileTransferManager : public SigC::Object
{
public:
    class FileTransferListener
    {
    public:
        virtual void transfer_update(int sz) = 0;
        virtual void transfer_closed() = 0;
        virtual void transfer_error(const std::string& msg) = 0;
    };

    typedef std::list<FileTransferListener*> ListenerList;
public:
    FileTransferManager();
    ~FileTransferManager();

    /** Initiate a transfer of the file to the specified jid.
     * @param jid The jid to send the file to
     * @param filename The actual system file to send
     * @returns string The id for the transfer
     */
    std::string initiate(const std::string& jid, const std::string& filename,
                         FTProfile::FileInfo& info);
    void receive(const std::string& jid, const std::string& filename,
                 Stream* stream, const std::string& id);
    void stop(const std::string& sid);
    void add_listener(const std::string& id, FileTransferListener* listener);
private:
    struct Transfer
    {
        std::string id;
        std::string jid;
        std::string filename;
        std::fstream file; 
        Stream* stream;
        ListenerList listeners;
        bool stop;
        FTProfile::FileInfo info;
    };

    typedef std::map<std::string, Transfer*> TransfersMap;
private:
    TransfersMap _transfers;

    // Initiation callbacks
    void on_disco_node(const jabberoo::DiscoDB::Item* item, Transfer* transfer);
    void on_si_result_node(const judo::Element& elem, Transfer* transfer);
    void on_initiator_state_changed(Stream::State state, Transfer* transfer);
    void on_initiator_data_sent(int sz, Transfer* transfer);
    void on_initiator_can_send_more(Transfer* transfer);

    // Receiver callbacks
    void on_receiver_data_available(const char* data, int sz, 
                                    Transfer* transfer);

    // Common callbacks
    void on_transfer_closed(Transfer* transfer);

    // Other functions
    void transfer_error(Transfer* transfer, const std::string& errmsg);

};

};
#endif //INCL_FILETRANSFERMANAGER_HH
