/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * 
 *  Gabber 2
 *  Based on Gabber, Copyright (c) 1999-2002 Dave Smith & Julian Missig
 *  Copyright (c) 2002 Julian Missig
 */

#include "LogManager.hh"
#include "XPathLog.hh"
#include <glibmm/miscutils.h>
#include <glibmm/fileutils.h>
#include <JID.hh>
#include <sys/stat.h>
#include <sys/types.h>

using namespace Gabber;

LogManager::LogManager() :
    _home_dir(Glib::build_filename(Glib::get_home_dir(),".Gabber"))
{
    // Make the gabber dir if it doesn't exist
    if (!Glib::file_test(_home_dir, 
        Glib::FILE_TEST_EXISTS | Glib::FILE_TEST_IS_DIR))
    {
        mkdir(_home_dir.c_str(), 0700);
    }
    std::string debug_path = Glib::build_filename(_home_dir, "debug.log");
    _debug_log.open(debug_path.c_str());
}

LogManager::~LogManager()
{
    for(LogMap::iterator i = _logs.begin(); i != _logs.end(); ++i)
    {
        i->second.first->close();
        delete i->second.first;
    }
    _debug_log.close();
}

// XXX Time stamp option for debug logs?
std::ostream& LogManager::debug()
{ return _debug_log; }

std::ostream& LogManager::log(const std::string& jid)
{
    LogMap::iterator it = _logs.find(jid);
    // If we don't have this log yet we need to remove one and add this one
    if (it != _logs.end())
    {
        return *it->second.first;

    }
    // XXX make this not hardcoded?
    if (_logs.size() > 5)
    {
        LogMap::iterator max = _logs.begin();
        for(LogMap::iterator i = _logs.begin(); i != _logs.end(); ++i)
        {
            if (i->second.second > max->second.second)
            {
                max = i;
            }
        }
        max->second.first->close();
        delete max->second.first;
        _logs.erase(max);
    }

    // setup and insert the new one
    std::string log_path = Glib::build_filename(_home_dir, 
    jabberoo::JID::getUserHost(jid));
    std::ofstream* log = new 
        std::ofstream(log_path.c_str(), std::ios::app);
    _logs.insert(LogMap::value_type(jid, std::make_pair(log, time(NULL))));

    return *log;
}

XPathLog* LogManager::create_xpath_log(const std::string& xpath, 
                                       const std::string& jid, bool in)
{
    return new XPathLog(*this, xpath, jid, in);
}
